# coding=utf-8
from datetime import date
from django.http import HttpResponse
from django.conf import settings
from apps.comun.constantes import MSG_NOT_AJAX
import os
import json
import sys
import logging

logger = logging.getLogger("matrizip")

#Importar wrapper constructor de cadenas
extapps_path = os.path.join(settings.PATH, "apis/chains/")

if not extapps_path in sys.path:
    sys.path.insert(1, extapps_path)
del extapps_path
from apis.chains import _svg_activities_builder

def generar_grafico_prod(request):
    """!
    Función que permite generar el gráfico de la cadena productiva a partir de un código de producción
    @author Ing. Roldan D. Vargas G. rvargas@cenditel.gob.ve
    @author Centro Nacional de Desarrollo e Investigación en Tecnologías Libres (CENDITEL) nodo Mérida
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 28-07-2014
    @pre Requiere auntenticación del usuario y selección de eslabon principal del cual generar la cadena
    @param[in] request  Variable de tipo object que contiene las peticiones del sistema
    @return Devuelve respuesta al sistema con los parámetros que permiten visualizar la ventana emergente sobre la
    cual mostrar el gráfico de la cadena de producción
    """
    try:
        if not request.is_ajax():
            return HttpResponse(json.dumps({'resultado': False, 'error': MSG_NOT_AJAX}))

        codigo = request.GET.get('rif', None)
        narriba = request.GET.get('narriba', None)
        nabajo = request.GET.get('nabajo', None)

        if codigo and narriba and nabajo:
            comando = "cd %s;%s --host=%s --port=%s --username=%s --password=%s --dbname=%s --levelsup=%s " \
                      "--levelsdown=%s --output=%s PROD %s"

            gen_graph = _svg_activities_builder.buildSvg(int(narriba),int(nabajo),
                                                         settings.DATABASES['default']['HOST'],
                                                         settings.DATABASES['default']['PORT'],
                                                         settings.DATABASES['default']['NAME'],
                                                         settings.DATABASES['default']['USER'],
                                                         settings.DATABASES['default']['PASSWORD'],
                                                         os.path.join(settings.PATH, "tmp/")+str(request.user),
                                                         "PROD",str(codigo))

            ahora = date.today().isoformat()

            logger.info(
                u"Se ha generado un nuevo gráfico de la matriz insumo / producto con el código de producción %s "
                u"solicitado por el usuario %s" % (codigo, str(request.user))
            )

            return HttpResponse(json.dumps({'resultado': True, 'ahora': ahora, 'arriba': narriba, 'abajo': nabajo,
                                            'usuario': str(request.user), 'varpk': 'codigo'}))

        else:
            return HttpResponse(json.dumps({'resultado': False, 'error': 'Debe indicar los parámetros necesarios '
                                                                         'para generar el gráfico solicitado'}))

    except Exception, e:
        logger.error(
            u"Ocurrió un error al generar el gráfico de la matriz insumo / producto solicitado por el usuario %s del "
            u"código de producción %s. Detalles: %s" % (str(request.user), request.GET.get('rif', ''), str(e))
        )
        return HttpResponse(json.dumps({'resultado': False, 'error': e}))


def generar_grafico_act(request):
    """!
    Función que permite generar el gráfico de la cadena productiva a partir de un código de actividad
    @author Ing. Roldan D. Vargas G. rvargas@cenditel.gob.ve
    @author Centro Nacional de Desarrollo e Investigación en Tecnologías Libres (CENDITEL) nodo Mérida
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 28-07-2014
    @pre Requiere auntenticación del usuario y selección de eslabon principal del cual generar la cadena
    @param[in] request  Variable de tipo object que contiene las peticiones del sistema
    @return Devuelve respuesta al sistema con los parámetros que permiten visualizar la ventana emergente sobre la
    cual mostrar el gráfico de la cadena de producción según la actividad seleccionada
    """
    try:
        if not request.is_ajax():
            return HttpResponse(json.dumps({'resultado': False, 'error': MSG_NOT_AJAX}))

        codigo = request.GET.get('rif', None)
        narriba = request.GET.get('narriba', None)
        nabajo = request.GET.get('nabajo', None)

        if codigo and narriba and nabajo:

            gen_graph = _svg_activities_builder.buildSvg(int(narriba),int(nabajo),
                                                         settings.DATABASES['default']['HOST'],
                                                         settings.DATABASES['default']['PORT'],
                                                         settings.DATABASES['default']['NAME'],
                                                         settings.DATABASES['default']['USER'],
                                                         settings.DATABASES['default']['PASSWORD'],
                                                         os.path.join(settings.PATH, "tmp/")+str(request.user),
                                                         "ACT",str(codigo))

            ahora = date.today().isoformat()

            logger.info(
                u"Se ha generado un nuevo gráfico de la matriz insumo / producto de la actividad económica %s "
                u"solicitada por el usuario %s" % (codigo, str(request.user))
            )

            return HttpResponse(json.dumps({'resultado': True, 'ahora': ahora, 'arriba': narriba, 'abajo': nabajo,
                                            'usuario': str(request.user), 'varpk': 'codigo'}))

        else:
            return HttpResponse(json.dumps({'resultado': False, 'error': 'Debe indicar los parámetros necesarios '
                                                                         'para generar el gráfico solicitado'}))

    except Exception, e:
        logger.error(
            u"Ocurrió un error al generar el gráfico de la matriz insumo / producto para la actividad económica %s "
            u"solicitado por el usuario %s. Detalles: %s" % (request.GET.get('rif', ''), str(request.user), str(e))
        )
        return HttpResponse(json.dumps({'resultado': False, 'error': e}))