# coding=utf-8
"""
Simulador Integral de Cadenas Productivas (SICP)

Copyleft (@) 2015 CENDITEL nodo Mérida - https://miv.cenditel.gob.ve/simulacion/
"""
## @package apps.simulacion.matrizip.views
#
# Clases y funciones a implementar en las vistas del módulo de consulta de la matriz insumo / producto
# @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
# @author <a href='http://www.cenditel.gob.ve'>Centro Nacional de Desarrollo e Investigación en Tecnologías Libres
# (CENDITEL) nodo Mérida - Venezuela</a>
# @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
# @date 07-10-2015
# @version 3.0.0

## Muestra el autor del script cuando es invocado
__author__ = "Ing. Roldan Vargas (rvargas at cenditel.gob.ve)"
## Muestra documentación breve sobre el script
__doc__    = "Clases y funciones a implementar en las vistas del módulo de consulta de la matriz insumo / producto"

from django.template.context import RequestContext
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.db.models import Sum
from django.db.models.loading import get_model
from apps.comun.functions import render_cadena
from apps.comun.models import CodigoAran
from apps.comun.constantes import MSG_NOT_AJAX
from apps.simulacion.models import ConvertirActividadACIIU, ConvertirCodigoArancelario
from apps.simulacion.sigesic.models import UnidadEconomica, Producto
from apps.simulacion.matrizip.models import ProduccionMIP, UtilizacionMIP, ActividadesImportacion, \
    Productos as ProductosMIP
import os
import json
import logging

logger = logging.getLogger("matrizip")

def graficar_matriz(request):
    """!
    Función que ejecuta la instrucción necesaria para mostrar el correspondiente gráfico de la Matriz Insumo/Producto
    asociada al escenario en estudio

    @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 19-10-2015
    @param request <b>{object}</b> Objeto que contiene la petición
    @return Devuelve el correspondiente gráfico de la matriz insumo/producto a mostrar
    """
    rif = request.GET['rif']
    id_prod = request.GET['id_prod']
    ahora = request.GET.get('ahora')

    existe, cod_act = None, None

    pr = Producto.objects.using('sigesic').get(pk=id_prod)

    try:
        cod_aran = CodigoAran.objects.get(pk=pr.codigo_aran_id)
        ca = ConvertirCodigoArancelario.objects.filter(codigo=cod_aran)

        if ca:
            cod_pr = ca[0].codigo_producto
            if cod_pr.__len__() == 1:
                cod_pr = "00%s" % cod_pr
            elif cod_pr.__len__() == 2:
                cod_pr = "0%s" % cod_pr

            if ProduccionMIP.objects.using('matrizip_1997').filter(codigo_producto=cod_pr):
                existe = True

            if existe:
                salida = os.path.join(settings.BASE_DIR, "tmp/%s_%s.svg" % (str(request.user),ahora))

                narriba = '2'
                nabajo = '2'
                os.system("cd " + os.path.join(settings.BASE_DIR, "tmp/"))
                os.system(
                    os.path.join(settings.BASE_DIR, "apis/chains/print_prod_chain_act") +
                    " --host=" + settings.DATABASES['matrizip_1997']['HOST'] +
                    " --port=" + settings.DATABASES['matrizip_1997']['PORT'] +
                    " --username=" + settings.DATABASES['matrizip_1997']['USER'] +
                    " --password=" +settings.DATABASES['matrizip_1997']['PASSWORD'] +
                    " --dbname=" + settings.DATABASES['matrizip_1997']['NAME'] +
                    " --levelsup=" + narriba + " --levelsdown=" + nabajo +
                    " --output=" + str(request.user) + "_" + ahora + " PROD " + cod_pr
                )
                archivo = str(request.user) + "_" + ahora + ".svg "
                os.system(
                    "mv -f " + os.path.join(settings.BASE_DIR, archivo) + os.path.join(settings.BASE_DIR, "tmp/")+";"+
                    "chmod -R 755 "+ os.path.join(settings.BASE_DIR, "tmp/")
                )

                cadena = render_cadena(str(request.user))

                logger.info(
                    u"Se ha generado un nuevo archivo con el gráfico de la matriz insumo / producto solicitada por el "
                    u"usuario %s del producto %s" % (str(request.user), cod_pr)
                )
    except Exception, e:
        logger.error(
            u"Ocurrió un error al generar el archivo de imagen de la matriz insumo / producto solicitada por el "
            u"usuario %s del producto %s. Detalles: %s" % (str(request.user), id_prod, str(e))
        )

    return render_to_response("grafico.html", {
        "username":request.user, "existe":existe,"rif":rif,'ahora':ahora, 'matriz':True, 'prod':pr
    }, context_instance=RequestContext(request))


def datos_produccion(request):
    """!
    Función que ejecuta la instrucción necesaria para mostrar datos de producción, utilización e importación de la
    Matriz Insumo / Producto asociada al escenario en estudio

    @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 27-10-2015
    @param request <b>{object}</b> Objeto que contiene la petición
    @return Devuelve un JSON con los datos de la Matriz Insumo / Producto
    """
    try:
        if not request.is_ajax():
            return HttpResponse(json.dumps({'resultado': False, 'error': MSG_NOT_AJAX}))

        id_prod = request.GET.get('id_prod', None)
        rif = request.GET.get('rif', None)

        if not id_prod or not rif:
            return HttpResponse(json.dumps({
                'resultado': False, 'error': _(u"No existe datos de producción en la matriz insumo / producto.")
            }))

        datos = {
            'produccion': [],
            'utilizacion': [],
            'importacion': []
        }

        ue = UnidadEconomica.objects.using('sigesic').get(rif=rif)
        ae_ciiu = ue.actividadciiu_set.get(activo=True, principal=True)
        actividad = {'codigo': ae_ciiu.ciiu_id, 'descripcion': ae_ciiu.ciiu.descripcion}
        pr = Producto.objects.using('sigesic').get(pk=id_prod)
        cod_aran = CodigoAran.objects.get(pk=pr.codigo_aran_id)
        ca = ConvertirCodigoArancelario.objects.get(codigo=cod_aran)

        produccion_total, utilizacion_total, importacion_total = 0, 0, 0

        if ca:
            cod_pr = ca.codigo_producto
            for prdlen in range(1,3):
                if cod_pr.__len__() <= prdlen:
                    cod_pr = "0%s" % cod_pr


            actividades = lambda x, y: [
                p_mip.codigo_actividad_id
                for p_mip in get_model('matrizip', y).objects.using('matrizip_1997').filter(codigo_producto=x)
            ]
            productos = lambda x, y: [
                p.codigo_producto_id
                for p in get_model('matrizip', y).objects.using('matrizip_1997').filter(codigo_actividad__in=x).distinct('codigo_producto')
            ]
            produccion_total = ProduccionMIP.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'ProduccionMIP'),'ProduccionMIP')).aggregate(total_produccion=Sum('prod_mill_bs'))
            for prdmip in ProduccionMIP.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'ProduccionMIP'),'ProduccionMIP')).distinct('codigo_producto'):
                for p in ProduccionMIP.objects.using('matrizip_1997').filter(codigo_producto=prdmip.codigo_producto_id).values('codigo_producto').annotate(total_produccion=Sum('prod_mill_bs')):
                    datos['produccion'].append(
                        {
                            'total_produccion': str(p['total_produccion']),
                            'codigo_producto': str(p['codigo_producto']),
                            'descripcion': ProductosMIP.objects.using('matrizip_1997').get(codigo_producto=p['codigo_producto']).descripcion,
                            'porcentaje': str(round((p['total_produccion'] * 100) / produccion_total['total_produccion'], 2))
                        }
                    )

            utilizacion_total = UtilizacionMIP.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'UtilizacionMIP'),'UtilizacionMIP')).aggregate(total_utilizacion=Sum('util_mill_bs'))
            for utlmip in UtilizacionMIP.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'UtilizacionMIP'),'UtilizacionMIP')).distinct('codigo_producto'):
                for u in UtilizacionMIP.objects.using('matrizip_1997').filter(codigo_producto=utlmip.codigo_producto_id).values('codigo_producto').annotate(total_utilizacion=Sum('util_mill_bs')):
                    datos['utilizacion'].append(
                        {
                            'total_utilizacion': str(u['total_utilizacion']),
                            'codigo_producto': str(u['codigo_producto']),
                            'descripcion': ProductosMIP.objects.using('matrizip_1997').get(codigo_producto=u['codigo_producto']).descripcion,
                            'porcentaje': str(round((u['total_utilizacion'] * 100) / utilizacion_total['total_utilizacion'], 2))
                        }
                    )

            importacion_total = ActividadesImportacion.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'ActividadesImportacion'),'ActividadesImportacion')).aggregate(total_importacion=Sum('monto_mill_bs'))
            for imp in ActividadesImportacion.objects.using('matrizip_1997').filter(codigo_producto__in=productos(actividades(cod_pr,'ActividadesImportacion'),'ActividadesImportacion')).distinct('codigo_producto'):
                for i in ActividadesImportacion.objects.using('matrizip_1997').filter(codigo_producto=imp.codigo_producto_id).values('codigo_producto').annotate(total_importacion=Sum('monto_mill_bs')):
                    datos['importacion'].append(
                        {
                            'total_importacion': str(i['total_importacion']),
                            'codigo_producto': str(i['codigo_producto']),
                            'descripcion': ProductosMIP.objects.using('matrizip_1997').get(codigo_producto=i['codigo_producto']).descripcion,
                            'porcentaje': str(round((i['total_importacion'] * 100) / importacion_total['total_importacion'], 2))
                        }
                    )

        logger.info(
            u"Se han generado con éxito los datos de producción de la matriz insumo / producto solicitados por el "
            u"usuario %s para el producto %s" % (str(request.user), pr.nombre)
        )

        return HttpResponse(json.dumps({
            'resultado': True, 'ae_ciiu': actividad, 'produccion': datos['produccion'],
            'produccion_total': str(produccion_total['total_produccion']),
            'utilizacion': datos['utilizacion'], 'utilizacion_total': str(utilizacion_total['total_utilizacion']),
            'importacion': datos['importacion'], 'importacion_total': str(importacion_total['total_importacion'])
        }))

    except Exception, e:
        logger.error(
            u"Ocurrió un error al procesar la solicitud del usuario %s para los datos de producción de la "
            u"matriz insumo / producto. Detalles: %s" % (str(request.user), str(e))
        )
        return HttpResponse(json.dumps({'resultado': False, 'error': e}))